@@ -2,17 +2,17 @@
 ===============================================================================
 
   FILE:  lasinterval.cpp
-
+  
   CONTENTS:
-
+  
     see corresponding header file
-
+  
   PROGRAMMERS:
-
+  
     martin.isenburg@rapidlasso.com  -  http://rapidlasso.com
-
+  
   COPYRIGHT:
-
+  
     (c) 2011-2015, martin isenburg, rapidlasso - fast tools to catch reality
 
     This is free software; you can redistribute and/or modify it under the
@@ -21,14 +21,11 @@
 
     This software is distributed WITHOUT ANY WARRANTY and without even the
     implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-
+  
   CHANGE HISTORY:
-
-    24 December 2016 -- by Jean-Romain Roussel -- L51 changed && by || to enable compiling with clang++ on Linux
-    20 December 2016 -- by Jean-Romain Roussel -- Change fprint(stderr, ...), raise an exeption
-
+  
     see corresponding header file
-
+  
 ===============================================================================
 */
 #include "lasinterval.hpp"
@@ -38,28 +35,27 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-
-#if defined(__clang__)
-  #include <string>
-#else
-  #include <string.h>
-#endif
-
+#include <string.h>
 #include <assert.h>
-#include <stdexcept>
 
 #include <map>
 #include <set>
 using namespace std;
 
 #ifdef UNORDERED
-  // Check if on OS X and using cland (unordered map isn't part of tr1 namespace)
-  #if defined(__APPLE__) || defined(__clang__)
-    #include <unordered_map>
-  #else
-    #include <tr1/unordered_map>
+   // Figure out whether <unordered_map> is in tr1
+#  ifdef __has_include
+#    if __has_include(<unordered_map>)
+#     include <unordered_map>
+      using namespace std;
+#     define UNORDERED_FOUND
+#    endif
+#  endif
+#  ifndef UNORDERED_FOUND
+#    include <tr1/unordered_map>
+    using namespace std;
     using namespace tr1;
-  #endif
+#   endif
 typedef unordered_map<I32, LASintervalStartCell*> my_cell_hash;
 #else
 #include <hash_map>
@@ -241,7 +237,7 @@ void LASinterval::merge_intervals(U32 maximum_intervals, const BOOL verbose)
   // maybe nothing to do
   if (map.size() <= maximum_intervals)
   {
-    if (verbose) throw std::runtime_error(std::string("next largest interval gap is %u")); //diff
+    if (verbose) REprintf("next largest interval gap is %u\n", diff);
     return;
   }
 
@@ -287,7 +283,7 @@ void LASinterval::merge_intervals(U32 maximum_intervals, const BOOL verbose)
     }
     map_element++;
   }
-  throw std::runtime_error(std::string("largest interval gap increased to %u")); //diff
+  REprintf("largest interval gap increased to %u\n", diff);
 
   // update totals
 
@@ -564,25 +560,25 @@ BOOL LASinterval::read(ByteStreamIn* stream)
   char signature[4];
   try { stream->getBytes((U8*)signature, 4); } catch (...)
   {
-    throw std::runtime_error(std::string("ERROR (LASinterval): reading signature"));
+    REprintf("ERROR (LASinterval): reading signature\n");
     return FALSE;
   }
   if (strncmp(signature, "LASV", 4) != 0)
   {
-    throw std::runtime_error(std::string("ERROR (LASinterval): wrong signature %4s instead of 'LASV'")); //signature
+    REprintf("ERROR (LASinterval): wrong signature %4s instead of 'LASV'\n", signature);
     return FALSE;
   }
   U32 version;
   try { stream->get32bitsLE((U8*)&version); } catch (...)
   {
-    throw std::runtime_error(std::string("ERROR (LASinterval): reading version"));
+    REprintf("ERROR (LASinterval): reading version\n");
     return FALSE;
   }
   // read number of cells
   U32 number_cells;
   try { stream->get32bitsLE((U8*)&number_cells); } catch (...)
   {
-    throw std::runtime_error(std::string("ERROR (LASinterval): reading number of cells"));
+    REprintf("ERROR (LASinterval): reading number of cells\n");
     return FALSE;
   }
   // loop over all cells
@@ -592,7 +588,7 @@ BOOL LASinterval::read(ByteStreamIn* stream)
     I32 cell_index;
     try { stream->get32bitsLE((U8*)&cell_index); } catch (...)
     {
-      throw std::runtime_error(std::string("ERROR (LASinterval): reading cell index"));
+      REprintf("ERROR (LASinterval): reading cell index\n");
       return FALSE;
     }
     // create cell and insert into hash
@@ -603,14 +599,14 @@ BOOL LASinterval::read(ByteStreamIn* stream)
     U32 number_intervals;
     try { stream->get32bitsLE((U8*)&number_intervals); } catch (...)
     {
-      throw std::runtime_error(std::string("ERROR (LASinterval): reading number of intervals in cell"));
+      REprintf("ERROR (LASinterval): reading number of intervals in cell\n");
       return FALSE;
     }
     // read number of points in cell
     U32 number_points;
     try { stream->get32bitsLE((U8*)&number_points); } catch (...)
     {
-      throw std::runtime_error(std::string("ERROR (LASinterval): reading number of points in cell"));
+      REprintf("ERROR (LASinterval): reading number of points in cell\n");
       return FALSE;
     }
     start_cell->full = number_points;
@@ -620,13 +616,13 @@ BOOL LASinterval::read(ByteStreamIn* stream)
       // read start of interval
       try { stream->get32bitsLE((U8*)&(cell->start)); } catch (...)
       {
-        throw std::runtime_error(std::string("ERROR (LASinterval): reading start %d of interval")); //cell->start
+        REprintf("ERROR (LASinterval): reading start %d of interval\n", cell->start);
         return FALSE;
       }
       // read end of interval
       try { stream->get32bitsLE((U8*)&(cell->end)); } catch (...)
       {
-        throw std::runtime_error(std::string("ERROR (LASinterval): reading end %d of interval")); //cell->end
+        REprintf("ERROR (LASinterval): reading end %d of interval\n", cell->end);
         return FALSE;
       }
       start_cell->total += (cell->end - cell->start + 1);
@@ -647,20 +643,20 @@ BOOL LASinterval::write(ByteStreamOut* stream) const
 {
   if (!stream->putBytes((U8*)"LASV", 4))
   {
-    throw std::runtime_error(std::string("ERROR (LASinterval): writing signature"));
+    REprintf("ERROR (LASinterval): writing signature\n");
     return FALSE;
   }
   U32 version = 0;
   if (!stream->put32bitsLE((U8*)&version))
   {
-    throw std::runtime_error(std::string("ERROR (LASinterval): writing version"));
+    REprintf("ERROR (LASinterval): writing version\n");
     return FALSE;
   }
   // write number of cells
   U32 number_cells = ((my_cell_hash*)cells)->size();
   if (!stream->put32bitsLE((U8*)&number_cells))
   {
-    throw std::runtime_error(std::string("ERROR (LASinterval): writing number of cells %d")); //number_cells
+    REprintf("ERROR (LASinterval): writing number of cells %d\n", number_cells);
     return FALSE;
   }
   // loop over all cells
@@ -680,19 +676,19 @@ BOOL LASinterval::write(ByteStreamOut* stream) const
     I32 cell_index = (*hash_element).first;
     if (!stream->put32bitsLE((U8*)&cell_index))
     {
-      throw std::runtime_error(std::string("ERROR (LASinterval): writing cell index %d")); //cell_index
+      REprintf("ERROR (LASinterval): writing cell index %d\n", cell_index);
       return FALSE;
     }
     // write number of intervals in cell
     if (!stream->put32bitsLE((U8*)&number_intervals))
     {
-      throw std::runtime_error(std::string("ERROR (LASinterval): writing number of intervals %d in cell")); //number_intervals
+      REprintf("ERROR (LASinterval): writing number of intervals %d in cell\n", number_intervals);
       return FALSE;
     }
     // write number of points in cell
     if (!stream->put32bitsLE((U8*)&number_points))
     {
-      throw std::runtime_error(std::string("ERROR (LASinterval): writing number of points %d in cell")); //number_points
+      REprintf("ERROR (LASinterval): writing number of points %d in cell\n", number_points);
       return FALSE;
     }
     // write intervals
@@ -702,13 +698,13 @@ BOOL LASinterval::write(ByteStreamOut* stream) const
       // write start of interval
       if (!stream->put32bitsLE((U8*)&(cell->start)))
       {
-        throw std::runtime_error(std::string("ERROR (LASinterval): writing start %d of interval")); //cell->start
+        REprintf("ERROR (LASinterval): writing start %d of interval\n", cell->start);
         return FALSE;
       }
       // write end of interval
       if (!stream->put32bitsLE((U8*)&(cell->end)))
       {
-        throw std::runtime_error(std::string("ERROR (LASinterval): writing end %d of interval")); //cell->end
+        REprintf("ERROR (LASinterval): writing end %d of interval\n", cell->end);
         return FALSE;
       }
       cell = cell->next;
